@@ -1,7 +1,10 @@
 #pragma once
 
 #include "ds/address.h"
+#include "ds/flaglock.h"
 #include "pal_plain.h"
+
+#include <array>
 #ifdef OPEN_ENCLAVE
 extern "C" void* oe_memset_s(void* p, size_t p_size, int c, size_t size);
 extern "C" [[noreturn]] void oe_abort();
@@ -10,8 +13,15 @@ namespace snmalloc
 {
   class PALOpenEnclave
   {
-    static inline std::atomic<void*> oe_base;
-    static inline void* oe_end = nullptr;
+    /// Base of OE heap
+    static inline void* heap_base = nullptr;
+
+    /// Size of OE heap
+    static inline size_t heap_size;
+
+    // This is infrequently used code, a spin lock simplifies the code
+    // considerably, and should never be on the fast path.
+    static inline std::atomic_flag spin_lock;
 
   public:
     /**
@@ -19,8 +29,8 @@ namespace snmalloc
      */
     static void setup_initial_range(void* base, void* end)
     {
-      oe_base = base;
-      oe_end = end;
+      heap_size = pointer_diff(base, end);
+      heap_base = base;
     }
 
     /**
@@ -37,23 +47,18 @@ namespace snmalloc
       oe_abort();
     }
 
-    template<bool committed>
-    void* reserve(size_t size) noexcept
+    static std::pair<void*, size_t>
+    reserve_at_least(size_t request_size) noexcept
     {
-      void* old_base = oe_base;
-      void* next_base;
-      auto end = oe_end;
-      do
-      {
-        auto new_base = old_base;
-        next_base = pointer_offset(new_base, size);
+      // First call returns the entire address space
+      // subsequent calls return {nullptr, 0}
+      FlagLock lock(spin_lock);
+      if (request_size > heap_size)
+        return {nullptr, 0};
 
-        if (next_base > end)
-          return nullptr;
-
-      } while (!oe_base.compare_exchange_strong(old_base, next_base));
-
-      return old_base;
+      auto result = std::make_pair(heap_base, heap_size);
+      heap_size = 0;
+      return result;
     }
 
     template<bool page_aligned = false>
